import { defineComponent, ref, shallowRef, computed, onMounted, onBeforeUnmount, watch, openBlock, createElementBlock, normalizeStyle } from 'vue';
import { PivotChart, PivotTable, ListTable } from '@visactor/vtable';
import { TABLE_EVENTS_KEYS, TABLE_EVENTS } from '../eventsUtils.js';

var _sfc_main = defineComponent({
    __name: 'base-table',
    props: {
        type: { type: String, required: false },
        options: { type: null, required: false },
        records: { type: Array, required: false },
        width: { type: [Number, String], required: false, default: '100%' },
        height: { type: [Number, String], required: false, default: '100%' },
        onReady: { type: Function, required: false },
        onError: { type: Function, required: false },
        onClickCell: { type: null, required: false },
        onDblClickCell: { type: null, required: false },
        onMouseDownCell: { type: null, required: false },
        onMouseUpCell: { type: null, required: false },
        onSelectedCell: { type: null, required: false },
        onKeyDown: { type: null, required: false },
        onMouseEnterTable: { type: null, required: false },
        onMouseLeaveTable: { type: null, required: false },
        onMouseDownTable: { type: null, required: false },
        onMouseMoveCell: { type: null, required: false },
        onMouseEnterCell: { type: null, required: false },
        onMouseLeaveCell: { type: null, required: false },
        onContextMenuCell: { type: null, required: false },
        onResizeColumn: { type: null, required: false },
        onResizeColumnEnd: { type: null, required: false },
        onChangeHeaderPosition: { type: null, required: false },
        onChangeHeaderPositionStart: { type: null, required: false },
        onChangeHeaderPositionFail: { type: null, required: false },
        onSortClick: { type: null, required: false },
        onFreezeClick: { type: null, required: false },
        onScroll: { type: null, required: false },
        onDropdownMenuClick: { type: null, required: false },
        onMouseOverChartSymbol: { type: null, required: false },
        onDragSelectEnd: { type: null, required: false },
        onDropdownIconClick: { type: null, required: false },
        onDropdownMenuClear: { type: null, required: false },
        onTreeHierarchyStateChange: { type: null, required: false },
        onShowMenu: { type: null, required: false },
        onHideMenu: { type: null, required: false },
        onIconClick: { type: null, required: false },
        onLegendItemClick: { type: null, required: false },
        onLegendItemHover: { type: null, required: false },
        onLegendItemUnHover: { type: null, required: false },
        onLegendChange: { type: null, required: false },
        onMouseEnterAxis: { type: null, required: false },
        onMouseLeaveAxis: { type: null, required: false },
        onCheckboxStateChange: { type: null, required: false },
        onRadioStateChange: { type: null, required: false },
        onAfterRender: { type: null, required: false },
        onInitialized: { type: null, required: false },
        onPivotSortClick: { type: null, required: false },
        onDrillMenuClick: { type: null, required: false },
        onVChartEventType: { type: null, required: false },
        onChangeCellValue: { type: null, required: false },
        onMousedownFillHandle: { type: null, required: false },
        onDragFillHandleEnd: { type: null, required: false },
        onDblclickFillHandle: { type: null, required: false },
        onScrollVerticalEnd: { type: null, required: false },
        onScrollHorizontalEnd: { type: null, required: false }
    },
    emits: TABLE_EVENTS_KEYS,
    setup(__props, { expose: __expose, emit: __emit }) {
        const props = __props;
        const vTableContainer = ref(null);
        const vTableInstance = shallowRef(null);
        __expose({ vTableInstance });
        const containerWidth = computed(() => (typeof props.width === 'number' ? `${props.width}px` : props.width));
        const containerHeight = computed(() => (typeof props.height === 'number' ? `${props.height}px` : props.height));
        const emit = __emit;
        const bindEvents = (instance) => {
            TABLE_EVENTS_KEYS.forEach(eventKey => {
                const vueEventHandler = (event) => {
                    emit(eventKey, event);
                };
                instance.on(TABLE_EVENTS[eventKey], vueEventHandler);
            });
        };
        const createTableInstance = (Type, options) => {
            vTableInstance.value = new Type(vTableContainer.value, options);
        };
        const createVTable = () => {
            var _a, _b;
            if (!vTableContainer.value) {
                return;
            }
            if (vTableInstance.value) {
                vTableInstance.value.release();
            }
            const getRecords = () => {
                return props.records !== undefined && props.records !== null && props.records.length > 0
                    ? props.records
                    : props.options.records;
            };
            try {
                switch (props.type) {
                    case 'list':
                        createTableInstance(ListTable, Object.assign(Object.assign({}, props.options), { records: getRecords() }));
                        break;
                    case 'pivot':
                        createTableInstance(PivotTable, Object.assign(Object.assign({}, props.options), { records: getRecords() }));
                        break;
                    case 'chart':
                        createTableInstance(PivotChart, Object.assign(Object.assign({}, props.options), { records: getRecords() }));
                        break;
                }
                bindEvents(vTableInstance.value);
                (_a = props.onReady) === null || _a === void 0 ? void 0 : _a.call(props, vTableInstance.value, true);
            }
            catch (err) {
                (_b = props.onError) === null || _b === void 0 ? void 0 : _b.call(props, err);
            }
        };
        const updateVTable = (newOptions) => {
            var _a;
            if (!vTableInstance.value) {
                return;
            }
            try {
                switch (props.type) {
                    case 'list':
                        if (vTableInstance.value instanceof ListTable) {
                            vTableInstance.value.updateOption(newOptions);
                        }
                        break;
                    case 'pivot':
                        if (vTableInstance.value instanceof PivotTable) {
                            vTableInstance.value.updateOption(newOptions);
                        }
                        break;
                    case 'chart':
                        if (vTableInstance.value instanceof PivotChart) {
                            vTableInstance.value.updateOption(newOptions);
                        }
                        break;
                }
            }
            catch (err) {
                (_a = props.onError) === null || _a === void 0 ? void 0 : _a.call(props, err);
            }
        };
        onMounted(createVTable);
        onBeforeUnmount(() => { var _a; return (_a = vTableInstance.value) === null || _a === void 0 ? void 0 : _a.release(); });
        watch(() => props.options, newOptions => {
            if (vTableInstance.value) {
                updateVTable(newOptions);
            }
            else {
                createVTable();
            }
        });
        watch(() => props.records, (newRecords, oldRecords) => {
            if (vTableInstance.value) {
                updateVTable(Object.assign(Object.assign({}, props.options), { records: newRecords }));
            }
            else {
                createVTable();
            }
        }, { deep: true });
        return (_ctx, _cache) => {
            return (openBlock(), createElementBlock("div", {
                ref_key: "vTableContainer",
                ref: vTableContainer,
                style: normalizeStyle([{ width: containerWidth.value, height: containerHeight.value }, { "position": "relative" }])
            }, null, 4));
        };
    }
});

export { _sfc_main as default };
